/*
 * @author norbert
 * @date 01.09.2006
 * @version $Id: PCWSClientExample.java,v 1.1 2006-09-01 13:52:46 norbert Exp $
 */
package com.gentics.portalnode.sdk;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.net.URL;

import org.apache.axis.client.Service;

import com.gentics.portalnode.pcws.example.DatasourceSoapBindingStub;
import com.gentics.portalnode.pcws.example.DatasourceSorting;
import com.gentics.portalnode.pcws.example.SimpleWSAttribute;
import com.gentics.portalnode.pcws.example.SimpleWSObject;

/**
 * Very simple WebService example with axis. <br>The Datasource WebService
 * classes have been autogenerated from the wsdl file using axis:
 * <pre>
 * java -cp mailapi.jar:activation.jar:saaj.jar:wsdl4j-1.5.1.jar:jaxrpc-1.2.1.jar:commons-discovery-0.2.jar:commons-logging-1.0.4.jar:axis-1.2.1.jar org.apache.axis.wsdl.WSDL2Java -p com.gentics.portalnode.pcws.example -U admin -P secret http://localhost:42880/Portal.Node/ws/services/datasource?wsdl
 * </pre>
 */
public class PCWSClientExample {

    /**
     * Main method. Fetch some objects from the webservice and print them to
     * stdout.
     * @param args
     */
    public static void main(String[] args) {
        try {
            // connect to the webservice
            Service service = new Service();
            URL url = new URL(
                    "http://localhost:42880/Portal.Node/ws/services/datasource/sample");
            DatasourceSoapBindingStub ds = new DatasourceSoapBindingStub(url, service);
            // set the user credentials
            ds.setUsername("admin");
            ds.setPassword("secret");

            // count the total available objects
            System.out.println("-- count objects --");
            System.out.println("Total of " + ds.getCount("") + " objects are available");

            // fetch the persons, sorted by surname
            System.out.println("-- get objects by rule --");
            SimpleWSObject[] objects = ds.getObjects("object.obj_type == 1001", new String[] {
                    "firstname", "surname"}, 0, -1,
                    new DatasourceSorting[] {new DatasourceSorting("surname", 1)});

            // print the objects with attributes
            showObjects(objects);

            // refetch the first object, but with more attributes
            System.out.println("-- get single object --");
            SimpleWSObject firstObject = ds.getObject(objects[0].getId(), new String[] {
                    "firstname", "surname", "address"});

            // print this object
            showObjects(new SimpleWSObject[] {firstObject});

            // fetch and show a single attribute
            System.out.println("-- get single Attribute --");
            showAttribute(ds.getAttribute(firstObject.getId(), "dateadded"));

            // fetch objects by ids (first and last object)
            System.out.println("-- get objects by ids --");
            SimpleWSObject[] twoObjects = ds.getObjectsByID(new String[] {firstObject.getId(),
                    objects[objects.length - 1].getId()}, new String[] {"firstname"}, 0, -1,
                    new DatasourceSorting[] {new DatasourceSorting("firstname", 2)});

            // print the objects
            showObjects(twoObjects);
        } catch (Exception ex) {
            // in case of error, just print the exception stacktrace
            ex.printStackTrace();
        }
    }

    /**
     * Helper method to show the given objects
     * @param objects array of objects
     * @throws Exception
     */
    public static void showObjects(SimpleWSObject[] objects) throws Exception {
        // run through all objects
        for (int i = 0; i < objects.length; i++) {
            // print the id
            System.out.println(objects[i].getId());

            // get all attributes
            SimpleWSAttribute[] attributes = objects[i].getAttributes();

            // run through the attributes
            for (int j = 0; j < attributes.length; j++) {
                SimpleWSAttribute attribute = attributes[j];
                showAttribute(attribute);
            }
        }
    }

    /**
     * Helper method to show the given attribute
     * @param attribute attribute to show
     * @throws Exception
     */
    public static void showAttribute(SimpleWSAttribute attribute) throws Exception {
        // get the attribute type
        String type = attribute.getType();

        // output the attribute name and type
        System.out.print("\t" + attribute.getName() + "(" + type + "): ");

        // get the getter method for the attributes type (e.g. when the type is
        // "stringValue", the getter is named "getStringValue").
        Method getter = attribute.getClass().getMethod(
                "get" + type.substring(0, 1).toUpperCase() + type.substring(1), null);

        // invoke the getter (get the attribute value)
        Object value = getter.invoke(attribute, null);

        if (value == null) {
            // empty value
            System.out.println();
        } else if (value.getClass().isArray()) {
            // array of values
            int length = Array.getLength(value);
            for (int i = 0; i < length; ++i) {
                if (i > 0) {
                    System.out.print(", ");
                }
                System.out.print(Array.get(value, i));
            }
            System.out.println();
        } else {
            // single value
            System.out.println(value);
        }
    }
}
